/*
	file automatically generated by make_test_files.pl
	Mon Apr  9 12:35:12 2001
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DARI
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_DARI
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_dari_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

int unur_dari_set_pedantic( struct unur_par *par, int pedantic );

static int COMPARE_SAMPLE_SIZE = 10000;
static int VIOLATE_SAMPLE_SIZE = 20;





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_dari_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_dari_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(7331)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("dari: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 33, (unur_dari_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 33, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 39, (unur_dari_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 39, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new();   /* no PMF */ 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 45, (unur_dari_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 45, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 55, (unur_dari_set_cpfactor(par,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 55, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 58, (unur_dari_set_squeeze(par, 1)) );
n_tests_failed += check_errorcode( TESTLOG, 58, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 61, (unur_dari_set_tablesize(par,100)) );
n_tests_failed += check_errorcode( TESTLOG, 61, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 64, (unur_dari_set_verify(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 64, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dgt_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 73, (unur_dari_set_cpfactor(par,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 73, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 76, (unur_dari_set_squeeze(par, 1)) );
n_tests_failed += check_errorcode( TESTLOG, 76, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 79, (unur_dari_set_tablesize(par,100)) );
n_tests_failed += check_errorcode( TESTLOG, 79, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 82, (unur_dari_set_verify(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 82, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dari_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 92, (unur_dari_set_cpfactor(par,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 92, UNUR_ERR_PAR_SET );

unur_errno = 0;
unur_dari_set_cpfactor(par,3.);
n_tests_failed += check_errorcode( TESTLOG, 95, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 98, (unur_dari_set_tablesize(par,-100)) );
n_tests_failed += check_errorcode( TESTLOG, 98, UNUR_ERR_PAR_SET );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dgt_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 115,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 118, (unur_dari_chg_verify( gen,1 )) );
n_tests_failed += check_errorcode( TESTLOG, 118, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 121, (unur_dari_chg_pmfparams( gen, fpar, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 121, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 124, (unur_dari_chg_domain( gen, 4, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 124, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 127, (unur_dari_chg_mode( gen, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 127, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 130, (unur_dari_chg_pmfsum( gen, 2. )) );
n_tests_failed += check_errorcode( TESTLOG, 130, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 133, (unur_dari_upd_mode( gen )) );
n_tests_failed += check_errorcode( TESTLOG, 133, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 136, (unur_dari_upd_pmfsum( gen )) );
n_tests_failed += check_errorcode( TESTLOG, 136, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dari_new(distr);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 143,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 147, (unur_dari_chg_pmfparams( gen, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 147, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 150, (unur_dari_chg_pmfparams( gen, fpar, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 150, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
unur_dari_chg_pmfparams( gen, fpar, 100 );
n_tests_failed += check_errorcode( TESTLOG, 153, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 156, (unur_dari_chg_domain( gen, 40, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 156, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 159, (unur_dari_chg_pmfsum( gen, -2. )) );
n_tests_failed += check_errorcode( TESTLOG, 159, UNUR_ERR_DISTR_SET );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* exist */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[1] = {0.5};
   distr = unur_distr_geometric(fpar,1);
   par = unur_dari_new(distr);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 173,    gen );

unur_errno = 0;
n_tests_failed += check_expected_reinit( TESTLOG, 176, (unur_dari_reinit( gen )) );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[] = {0.1};
   distr = unur_distr_geometric(fpar,1);
   par = NULL; 

unur_errno = 0;
/* default algorithm */
par = unur_dari_new(distr);
n_tests_failed += compare_int_sequence_par_start( TESTLOG, 189, urng, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* default algorithm - verifying mode */
par = unur_dari_new(distr);
unur_dari_set_verify(par,1);
n_tests_failed += compare_int_sequence_par( TESTLOG, 194, urng, par, COMPARE_SAMPLE_SIZE );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[15];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 15 */
fpm[0] = 0.5;
distr[0] = unur_distr_geometric(fpm,1);


fpm[0] = 0.1;
distr[1] = unur_distr_geometric(fpm,1);


fpm[0] = 0.001;
distr[2] = unur_distr_geometric(fpm,1);


fpm[0] = 0.1;
distr[3] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.9;
distr[4] = unur_distr_logarithmic(fpm,1);


fpm[0] = 0.5;
fpm[1] = 10.;
distr[5] = unur_distr_negativebinomial(fpm,2);


fpm[0] = 0.7;
fpm[1] = 5.;
distr[6] = unur_distr_negativebinomial(fpm,2);


fpm[0] = 0.1;
fpm[1] = 20.;
distr[7] = unur_distr_negativebinomial(fpm,2);


fpm[0] = 3.;
distr[8] = unur_distr_poisson(fpm,1);


fpm[0] = 50.;
distr[9] = unur_distr_poisson(fpm,1);


fpm[0] = 2.;
fpm[1] = 1.;
distr[10] = unur_distr_zipf(fpm,2);
unur_distr_discr_set_domain(distr[9],1,1000);


fpm[0] = 20.;
fpm[1] = 0.8;
distr[11] = unur_distr_binomial(fpm,2);


fpm[0] = 2000.;
fpm[1] = 0.0013;
distr[12] = unur_distr_binomial(fpm,2);


fpm[0] = 2000.;
fpm[1] = 200.;
fpm[2] = 20;
distr[13] = unur_distr_hypergeometric(fpm,3);


fpm[0] = 2000.;
fpm[1] = 200.;
fpm[2] = 220;
distr[14] = unur_distr_hypergeometric(fpm,3);


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 75 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("\n(verify hat) "); fflush(stdout);

/* verify hat tests: 75 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nVerify Hat Test (squeeze <= PDF <= hat):\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[0]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[1]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[2]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[3]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[4]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[5]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[6]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[7]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[8]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[9]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[11]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[12]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[13]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
	unur_dari_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 100);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 1);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_dari_new(distr[14]);
	unur_dari_set_pedantic(par,0);
unur_dari_set_tablesize(par, 0);
unur_dari_set_squeeze(par, 0);
	gen = unur_init(par);
	if (gen) unur_dari_chg_verify(gen,1);
	n_tests_failed += run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


/* no such function */
int unur_dari_set_pedantic( struct unur_par *par, int pedantic )
{ return 1; }



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_dari_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_DARI */
/*---------------------------------------------------------------------------*/
