/*
	file automatically generated by make_test_files.pl
	Mon Apr  9 12:35:11 2001
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DISTR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/
#ifdef T_DISTR
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static struct prng *urng;           /* uniform random number generator       */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_distr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

static int COMPARE_SAMPLE_SIZE = 500;
static int VIOLATE_SAMPLE_SIZE = 20;

double pdf(double x, UNUR_DISTR *distr);
double dpdf(double x, UNUR_DISTR *distr);
double cdf(double x, UNUR_DISTR *distr);

double pmf(int k, UNUR_DISTR *distr);
double icdf(int k, UNUR_DISTR *distr);

UNUR_DISTR *get_distr_with_pv( void );






/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_distr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_distr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

	/* set uniform random number generator */
	urng = prng_new("mt19937(7823)");
	unur_set_default_urng(urng);

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("distr: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 45, (unur_distr_set_name( distr, "nix" )) );
n_tests_failed += check_errorcode( TESTLOG, 45, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 48, (unur_distr_cont_set_pdf( distr, pdf )) );
n_tests_failed += check_errorcode( TESTLOG, 48, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 51, (unur_distr_cont_set_dpdf( distr, dpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 51, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 54, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 54, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 57, (unur_distr_cont_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 57, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 60, (unur_distr_cont_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 60, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 63, (unur_distr_cont_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 63, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 66, (unur_distr_cont_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 66, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 69, (unur_distr_discr_set_pv( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 69, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 72, (unur_distr_discr_set_pmf( distr, pmf )) );
n_tests_failed += check_errorcode( TESTLOG, 72, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 75, (unur_distr_discr_set_cdf( distr, icdf )) );
n_tests_failed += check_errorcode( TESTLOG, 75, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 78, (unur_distr_discr_set_pmfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 78, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 81, (unur_distr_discr_set_domain( distr, 0, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 81, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 84, (unur_distr_discr_set_mode( distr, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 84, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 87, (unur_distr_discr_set_pmfsum( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 87, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
	distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 93, (unur_distr_cont_set_pdf( distr, pdf )) );
n_tests_failed += check_errorcode( TESTLOG, 93, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 96, (unur_distr_cont_set_dpdf( distr, dpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 96, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 99, (unur_distr_cont_set_cdf( distr, cdf )) );
n_tests_failed += check_errorcode( TESTLOG, 99, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 102, (unur_distr_cont_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 102, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 105, (unur_distr_cont_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 105, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 108, (unur_distr_cont_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 108, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 111, (unur_distr_cont_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 111, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 117, (unur_distr_discr_set_pv( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 117, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 124, (unur_distr_discr_set_pmf( distr, pmf )) );
n_tests_failed += check_errorcode( TESTLOG, 124, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 127, (unur_distr_discr_set_cdf( distr, icdf )) );
n_tests_failed += check_errorcode( TESTLOG, 127, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 130, (unur_distr_discr_set_pmfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 130, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 133, (unur_distr_discr_set_domain( distr, 0, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 133, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 136, (unur_distr_discr_set_mode( distr, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 136, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 139, (unur_distr_discr_set_pmfsum( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 139, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 145, (unur_distr_cont_set_pdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 145, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 148, (unur_distr_cont_set_dpdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 148, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 151, (unur_distr_cont_set_cdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 151, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 154, (unur_distr_cont_set_pdfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 154, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 160, (unur_distr_discr_set_pmf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 160, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 163, (unur_distr_discr_set_cdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 163, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 166, (unur_distr_discr_set_pmfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 166, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double pv[] = {1.,2.,3.};
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 173, (unur_distr_cont_set_pdfparams( distr, pv, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 173, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 176, (unur_distr_cont_set_pdfparams( distr, pv, UNUR_DISTR_MAXPARAMS +1 )) );
n_tests_failed += check_errorcode( TESTLOG, 176, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 179, (unur_distr_cont_set_domain( distr, 0., -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 179, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 182, (unur_distr_cont_set_pdfarea( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 182, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double pv[] = {1.,2.,3.};
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 189, (unur_distr_discr_set_pmfparams( distr, pv, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 189, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 192, (unur_distr_discr_set_pmfparams( distr, pv, UNUR_DISTR_MAXPARAMS +1 )) );
n_tests_failed += check_errorcode( TESTLOG, 192, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 195, (unur_distr_discr_set_domain( distr, 10, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 198, (unur_distr_discr_set_pmfsum( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 198, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   double a, b;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 210, ((void*) unur_distr_get_name( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 210, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 213, (unur_distr_get_type( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 213, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 216, (unur_distr_is_cont( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 216, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 219, (unur_distr_is_discr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 219, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 222, ((void*) unur_distr_cont_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 222, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 225, ((void*) unur_distr_cont_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 225, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 228, ((void*) unur_distr_cont_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 228, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 231, (unur_distr_cont_get_pdfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 231, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 234, (unur_distr_cont_get_domain( distr, &a, &b )) );
n_tests_failed += check_errorcode( TESTLOG, 234, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 237, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 237, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 240, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 240, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 243, (unur_distr_discr_get_pv( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 243, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 246, ((void*) unur_distr_discr_get_pmf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 246, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 249, ((void*) unur_distr_discr_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 249, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 252, (unur_distr_discr_get_pmfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 252, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 255, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 255, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   double a,b;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 263, ((void*) unur_distr_cont_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 263, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 266, ((void*) unur_distr_cont_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 266, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 269, ((void*) unur_distr_cont_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 269, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 272, (unur_distr_cont_get_pdfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 272, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 275, (unur_distr_cont_get_domain( distr, &a, &b )) );
n_tests_failed += check_errorcode( TESTLOG, 275, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 278, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 278, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 281, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 281, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   int ia, ib;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 288, (unur_distr_discr_get_pv( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 288, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 291, (unur_distr_discr_get_domain( distr, &ia, &ib )) );
n_tests_failed += check_errorcode( TESTLOG, 291, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 299, ((void*) unur_distr_discr_get_pmf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 299, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 302, ((void*) unur_distr_discr_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 302, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 305, (unur_distr_discr_get_pmfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 305, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 308, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 308, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* parameters not unknwon */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 314, (unur_distr_cont_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 314, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 317, (unur_distr_cont_get_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 317, UNUR_ERR_DISTR_GET );
unur_distr_free(distr);
}

/* parameters not unknwon */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 323, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 323, UNUR_ERR_DISTR_GET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 334, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 334, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 337, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 337, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 340, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 340, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 346, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 346, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 349, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 349, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 355, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 355, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 361, (unur_distr_cont_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 361, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 364, (unur_distr_cont_upd_pdfarea( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 364, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 370, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 370, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 388, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 388, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 391, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 391, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 394, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 394, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 397, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 397, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 400, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 400, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 406, (unur_distr_cont_eval_pdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 406, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 409, (unur_distr_cont_eval_dpdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 409, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 412, (unur_distr_cont_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 412, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 418, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 418, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 421, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 421, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 1 */
distr[0] = unur_distr_normal(NULL,0);



/* free distributions */
	unur_distr_free(distr[0]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	if (n_tests_failed>0) n_tests_failed--;  /* we accept one failure */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf(" --> failed] ") : printf(" --> ok] ");

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


double pdf(double x, UNUR_DISTR *distr) { return 1.; }
double dpdf(double x, UNUR_DISTR *distr) { return 1.; }
double cdf(double x, UNUR_DISTR *distr) { return 1.; }

double pmf(int k, UNUR_DISTR *distr) { return 1.; }
double icdf(int k, UNUR_DISTR *distr) { return 1.; }

UNUR_DISTR *get_distr_with_pv( void )
{
#define PV_SIZE 200
  double pv[PV_SIZE];
  UNUR_DISTR *distr;
  int i;

  for (i=0; i<PV_SIZE; i++)
    pv[i] = prng_get_next(urng);

  distr = unur_distr_discr_new();
  unur_distr_discr_set_pv(distr,pv,PV_SIZE);

  return distr;

#undef PV_SIZE
} /* end of get_distr_with_pv() */



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_distr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_distr_set_verify(UNUR_PAR *par, int verify) {return 0;}
/*---------------------------------------------------------------------------*/
#else
/*---------------------------------------------------------------------------*/
int main() { exit(77); } /* ignore test */
/*---------------------------------------------------------------------------*/
#endif  /* T_DISTR */
/*---------------------------------------------------------------------------*/
